package org.sersar.cayenne.dao.impl;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.objectstyle.cayenne.map.DbAttribute;
import org.objectstyle.cayenne.map.DbEntity;
import org.objectstyle.cayenne.map.ObjAttribute;
import org.objectstyle.cayenne.map.ObjEntity;
import org.sersar.cayenne.S2DataContext;
import org.sersar.cayenne.S2DataContextFactory;
import org.sersar.cayenne.dao.DataContextUtil;

/**
 * @author Takahiro MATSUNAGA
 * @see AbstractCommand
 */
public abstract class AbstractSelectCommand extends AbstractCommand {

    private S2DataContextFactory s2DataContextFactory_;

    private String[] argNames_;

    private Method method_;

    private String query_;

    /**
     * @param s2DataContextFactory
     * @param beanClass
     */
    public AbstractSelectCommand(S2DataContextFactory s2DataContextFactory, Class beanClass) {
        super(beanClass);
        s2DataContextFactory_ = s2DataContextFactory;
    }

    /* ( Javadoc)
     * @see org.sersar.cayenne.dao.CayenneDaoCommand#execute(java.lang.Object[])
     */
    public Object execute(Object[] args) {
        S2DataContext s2DataContext = s2DataContextFactory_.getDataContext();
        ObjEntity objEntity = DataContextUtil.getObjEntity(s2DataContext, getBeanClass());

        setupSql(objEntity);
        setupParams(args);

        return getReturnObject(s2DataContext.performQuery(createTemplate(true)));
    }

    private void setupSql(ObjEntity objEntity) {
        StringBuffer sb = new StringBuffer("select");
 
        DbEntity dbEntity = objEntity.getDbEntity();
        Iterator entIt = dbEntity.getAttributes().iterator();
        while (entIt.hasNext()) {
            DbAttribute dbAttr = (DbAttribute) entIt.next();
            ObjAttribute objAttr = objEntity.getAttributeForDbAttribute(dbAttr);

            sb.append(" #result('");
            sb.append(objAttr.getName()).append("' '");
            sb.append(objAttr.getType()).append("'),");
        }
        sb.setLength(sb.length() - 1);

        sb.append(" from ");
        sb.append(objEntity.getDbEntityName());
        setupWhere(sb, objEntity);

        setSql(sb.toString());
    }

    protected abstract void setupWhere(StringBuffer sb, ObjEntity objEntity);

    /**
     * @return
     */
    private void setupParams(Object[] args) {
        if (args.length == 0) {
            return;
        }

        Map params = new HashMap();
        for (int i = 0; i < args.length; i++) {
            params.put(argNames_[i], args[i]);
        }

        setParams(params);
    }

    private Object getReturnObject(List ret){
        if (method_.getReturnType().isAssignableFrom(List.class)) {
            return ret;
        } else {
            if(ret == null || ret.size() == 0){
                return null;
            } else {
                return ret.get(0);
            }
        }
    }

    /**
     * @return argNames
     */
    public String[] getArgNames() {
        return argNames_;
    }

    /**
     * @param argNames argNames ݒ肵܂B
     */
    public void setArgNames(String[] argNames) {
        argNames_ = argNames;
    }
    
    public Method getMethod() {
        return method_;
    }

    public void setMethod(Method method) {
        method_ = method;
    }

    /**
     * @return query
     */
    public String getQuery() {
        return query_;
    }

    /**
     * @param query
     */
    public void setQuery(String query) {
        this.query_ = query;
    }

}
