package org.sersar.cayenne.impl;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.sql.DataSource;
import javax.transaction.Synchronization;
import javax.transaction.Transaction;
import javax.transaction.TransactionManager;

import org.objectstyle.cayenne.access.DataContext;
import org.objectstyle.cayenne.access.DataDomain;
import org.objectstyle.cayenne.access.DataNode;
import org.objectstyle.cayenne.conf.Configuration;
import org.objectstyle.cayenne.conf.DefaultConfiguration;
import org.objectstyle.cayenne.map.DataMap;
import org.seasar.framework.util.TransactionManagerUtil;
import org.seasar.framework.util.TransactionUtil;
import org.sersar.cayenne.S2DataContext;
import org.sersar.cayenne.S2DataContextFactory;
import org.sersar.cayenne.access.S2TransactionDelegate;

/**
 * @author Takahiro MATSUNAGA
 * @see S2DataContextFactory, Synchronization
 */
public class S2DataContextFactoryImpl implements S2DataContextFactory, Synchronization {

    private TransactionManager transactionManager_;

    private DataSource dataSource_;

    private String domainFile_ = "cayenne.xml";

    private Map txDataContexts_ = new HashMap();

    public S2DataContextFactoryImpl(TransactionManager transactionManager, DataSource dataSource) {
        transactionManager_ = transactionManager;
        dataSource_ = dataSource;
    }

    /**
     * @return transactionManager
     */
    public TransactionManager getTransactionManager() {
        return transactionManager_;
    }

    /**
     * @return dataSource
     */
    public DataSource getDataSource() {
        return dataSource_;
    }

    public int getTxDataContextSize() {
        return txDataContexts_.size();
    }

    /**
     * @return domainFile
     */
    public String getDomainFile() {
        return domainFile_;
    }

    /**
     * @param domainFile
     */
    public void setDomainFile(String domainFile) {
        this.domainFile_ = domainFile;
    }

    /* (Javadoc Ȃ)
     * @see org.sersar.cayenne.S2DataContextFactory#getDataContext()
     */
    public S2DataContext getDataContext() {
        Transaction tx = getTransaction();
        if (tx == null) {
            return createDataContext();
        }

        S2DataContext dataContext = (S2DataContext) txDataContexts_.get(tx);
        if (dataContext != null) {
            return dataContext;
        }

        return bindSession(tx);
    }

    /**
     * @return
     */
    private Transaction getTransaction() {
        return TransactionManagerUtil.getTransaction(getTransactionManager());
    }

    /**
     * @return
     */
    private S2DataContext createDataContext() {
        DataDomain domain = createDataDomain();
        DataContext dataContext = domain.createDataContext();

        return new S2DataContextImpl(dataContext);
    }

    /**
     * @return
     */
    private DataDomain createDataDomain() {
        Configuration.initializeSharedConfiguration(new DefaultConfiguration(domainFile_));
        DataDomain domain = Configuration.getSharedConfiguration().getDomain();
        domain.setTransactionDelegate(new S2TransactionDelegate());

        Iterator mapIt = domain.getDataMaps().iterator();
        while (mapIt.hasNext()) {
            DataMap dataMap = (DataMap) mapIt.next();
            DataNode dataNode = domain.lookupDataNode(dataMap);
            dataNode.setDataSource(dataSource_);
        }

        return domain;
    }

	/**
     * @param tx
     * @return
     */
    private synchronized S2DataContext bindSession(Transaction tx) {
        S2DataContext dataContext = (S2DataContext) txDataContexts_.get(tx);
        if (dataContext != null) {
            return dataContext;
        }
        dataContext = createDataContext();
        txDataContexts_.put(tx, dataContext);
        TransactionUtil.registerSynchronization(tx, this);

        return dataContext;
    }

    /* (Javadoc Ȃ)
     * @see javax.transaction.Synchronization#beforeCompletion()
     */
    public void beforeCompletion() {
        closeSession();
    }

    /* (Javadoc Ȃ)
     * @see javax.transaction.Synchronization#afterCompletion(int)
     */
    public void afterCompletion(int arg0) {
    }

    /**
     * 
     */
    private void closeSession() {
        Transaction tx = getTransaction();
        if (tx != null) {
            S2DataContext dataContext = null;
            synchronized (this) {
                dataContext = (S2DataContext) txDataContexts_.remove(tx);
            }
        }
    }

}
